@@ -18,5 +18,5 @@ void main()
         output();
     }
 
-  nport(plotlinreg("ms95", "plantphos", "soilphos", "bb", "", 1, 0.95), 4, 4);
+  nport(plotlinreg("ms95", "plantphos", "soilphos", "==", "", 1, 0.95), 4, 4);
 }
