@@ -1,6 +1,6 @@
 /* Contains the basic I/O and setup routines.  */
 
-/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  *  Dap is a GNU program.
  *
@@ -79,6 +79,7 @@ char *dap_psname;	/* name of the .ps file */
 
 extern int dap_dblhigh;	/* for coding doubles */
 extern int dap_dbllow;	/* for coding doubles */
+extern double dap_double;	/* to get around weird returns; see machdep.c */
 
 static int nmallocs = 0;
 static int nfrees = 0;
@@ -875,7 +876,10 @@ for (v = 0, l = 0; v < dap_obs[dap_ono].do_ivar; v++)
 	else if (dap_obs[dap_ono].do_len[dap_obs[dap_ono].do_in[v]] == DBL)
 		{
 		if (intype == DSET)
-			dap_obs[dap_ono].do_dbl[dap_obs[dap_ono].do_in[v]] = dap_getdouble(value);
+			{
+			dap_getdouble(value);
+			dap_obs[dap_ono].do_dbl[dap_obs[dap_ono].do_in[v]] = dap_double;
+			}
 		else
 			{
 			if (!i || (nfields && dap_blank(value)) || !strcmp(value, "."))
@@ -1475,7 +1479,7 @@ else if (!strcmp(action, "APPEND"))
 				if (dap_obs[0].do_len[onum] != baseobs.do_len[v])
 					{
 					fprintf(stderr,
-		"dataset: variable %s has different lengths (%d appended to %d) in datasets\n",
+		"(dataset) variable %s has different lengths (%d appended to %d) in datasets\n",
 					baseobs.do_nam[v], dap_obs[0].do_len[onum],
 					baseobs.do_len[v]);
 					exit(1);
@@ -2062,8 +2066,8 @@ for (v = 0, first = 1; v < dap_obs[dap_ono].do_ovar; v++)
 		if (!dap_ono && dap_obs[dap_ono].do_dl[dap_obs[0].do_out[v]])
 			dap_obs[0].do_dbl[dap_obs[0].do_out[v]] =
 				*dap_obs[dap_ono].do_dl[dap_obs[0].do_out[v]];
-		dap_putdouble(dap_obs[dap_ono].do_dbl[dap_obs[dap_ono].do_out[v]],
-				dap_out);
+		dap_double = dap_obs[dap_ono].do_dbl[dap_obs[dap_ono].do_out[v]];
+		dap_putdouble(dap_out);
 		}
 	else
 		{
