@@ -1,6 +1,6 @@
 /* dap1.c -- sort and table */
 
-/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -156,6 +156,11 @@ if (modifiers)
 			while (modifiers[l] == 'i' || modifiers[l] == 'd')
 				mod[nmods++] = modifiers[l++];
 			}
+		else
+			{
+			fprintf(dap_err, "(sort) Bad modifier(s): %s\n", modifiers);
+			exit(1);
+			}
 		while (modifiers[l] == ' ')
 			l++;
 		}
@@ -1073,7 +1078,7 @@ for (more = 1; more; )
 			if (nnan[v])
 				{
 				dap_swap();
-				fprintf(dap_log, "means: %d NaNs for %s\n",
+				fprintf(dap_log, "(means) %d NaNs for %s\n",
 					nnan[v], dap_obs[0].do_nam[varv[v]]);
 				dap_swap();
 				}
@@ -1687,7 +1692,7 @@ if (!tabinit)
 	}
 if (!fname)
 	{
-	fputs("table: no dataset name given\n", dap_err);
+	fputs("(table) no dataset name given\n", dap_err);
 	exit(1);
 	}
 inset(fname);
@@ -1695,12 +1700,12 @@ tabform[0] = '\0';
 rtitlesp = 8;
 if (!rowvars || !colvars)
 	{
-	fputs("table: no row and/or column variables specified\n", dap_err);
+	fputs("(table) no row and/or column variables specified\n", dap_err);
 	exit(1);
 	}
 if (!format)
 	{
-	fputs("table: no format given\n", dap_err);
+	fputs("(table) no format given\n", dap_err);
 	exit(1);
 	}
 /* NOTE: the following has a side-effect to change rtitlesp */
