@@ -4,7 +4,7 @@
  * some other time, folks.
  */
 
-/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -94,7 +94,7 @@ if (fabs(q) > 0.42)
 		r = 1.0 - p;
 	if (r <= 0.0)
 		{
-		fputs("zpoint: input not between 0 and 1\n", dap_err);
+		fputs("(zpoint) input not between 0 and 1\n", dap_err);
 		exit(1);
 		}
 	r = sqrt(-log(r));
