@@ -1,6 +1,6 @@
 /* ps1.c -- higher level functions */
 
-/*  Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+/*  Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,26 +35,6 @@ extern FILE *dap_err;
 
 extern char *pict_newstr(char *str);
 
-static double arintup(double x)
-{
-double i;
-
-if (fabs(i = ceil(x)) == 0.0)
-	return 0.0;
-else
-	return i;
-}
-
-static double arintdown(double x)
-{
-double i;
-
-if (fabs(i = floor(x)) == 0.0)
-	return 0.0;
-else
-	return i;
-}
-
 void pict_maketick(tick *t, double num, char *label, double len)
 {
 t->tick_num = num;
@@ -63,150 +43,152 @@ t->tick_len = len;
 }
 
 /* Add an y-axis to the picture pointed to by p.
- * ytick is the array of ticks to use, nyticks is the number, counting
+ * ytick is the array of ticks to use, nyticks is the number, not counting
  *	the axis label, which is the highest numbered "tick"
  * ypos is the y-coordinate of the y-axis, xpos is the y-coordinate of
  *	the x-axis: drawing ends at (ypos, xpos)
  * side = 1.0 for ticks extending to the right from the axis, -1.0 for left
  * marks indicates whether tick marks should be used
  */
-static void yaxis(pict *p, tick ytick[], int nyticks, double xpos, double ypos,
-			double side, int marks)
+static void yaxis(pict *p, double miny, double maxy, tick ytick[], int nyticks,
+			double xpos, double ypos, double side, int marks)
 {
 int ny;		/* index to ytick[] */
-char tpos[3];	/* text positioning string for axis label */
+char tpos[4];	/* text positioning string for axis label */
 char npos[3];	/* text positioning string for numerical values */
 double xlaboff;	/* x offset for label */
 double txtang;	/* for rotating the label if it lies along the axis */
 int lab1len;	/* string length of one label */
 int labslen;	/* maximum string length of labels */
 
-strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
+strcpy(tpos, "cb ");	/* coordinates give center, bottom point of axis label */
 if (side > 0.0)		/* if ticks extend to right */
 	strcpy(npos, "rm");	/* coordinates give right, middle point of number */
 else			/* if ticks extend to left */
 	strcpy(npos, "lm");	/* coordinates give left, middle point of number */
-for (labslen = 0, ny = 0; ny < nyticks - 1; ny++)
+for (labslen = 0, ny = 0; ny < nyticks; ny++)
 	{		/* find maximum string length of numerical labels */
 	lab1len = strlen(ytick[ny].tick_lab);
 	if (lab1len > labslen)
 		labslen = lab1len;
 	}
-ny = nyticks - 1;	/* index of axis label */
 txtang = 0.0;	/* used if label is at the top of the y-axis */
 /* check to see if axes label "tick" is between bottom and top "real" ticks,
  * so that axis label is along side the axis
  */
-if (ny >= 1 && ytick[0].tick_num < ytick[ny].tick_num &&
-		ytick[ny].tick_num < ytick[ny - 1].tick_num)
+if (miny <= ytick[nyticks].tick_num && ytick[nyticks].tick_num <= maxy)
 	{
 	/* have to increase distance for label to leave room for
 	 * numerical labels
 	 */
-	strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
-	xlaboff = 0.25 * p->pict_fs * ((double) (labslen + 3)) * ytick[ny].tick_len;
+	strcpy(tpos, "cb ");	/* coordinates give center, bottom point of axis label */
+	xlaboff = 0.25 * p->pict_fs * ((double) (labslen + 3)) * ytick[nyticks].tick_len;
 	txtang = side * 90.0;
 	}
-else if (ytick[ny].tick_num <= ytick[0].tick_num)	/* label at bottom */
+else if (ytick[nyticks].tick_num < miny)	/* label at bottom */
 	{
-	strcpy(tpos, "ct");	/* coordinates give center, top point of axis label */
+	strcpy(tpos, "ct ");	/* coordinates give center, top point of axis label */
 	xlaboff = 0.0;
 	}
 else							/* label at top */
 	{
-	strcpy(tpos, "cb");	/* coordinates give center, bottom point of axis label */
+	strcpy(tpos, "cb ");	/* coordinates give center, bottom point of axis label */
 	xlaboff = 0.0;
 	}
 /* for ticks extending to right or if extending to left and using marks */
 if (side > 0.0 || marks)
-	pict_text(p, ytick[ny].tick_lab, ypos - side * xlaboff,
-				ytick[ny].tick_num, txtang, tpos);
-while (--ny >= 0)
+	pict_text(p, ytick[nyticks].tick_lab, ypos - side * xlaboff,
+				ytick[nyticks].tick_num, txtang, tpos);
+pict_point(p, ypos, maxy);
+while (--nyticks >= 0)
 	{
-	pict_point(p, ypos, ytick[ny].tick_num);
-	pict_point(p, ypos + side * ytick[ny].tick_len, ytick[ny].tick_num);
-	pict_point(p, ypos, ytick[ny].tick_num);
+	pict_point(p, ypos, ytick[nyticks].tick_num);
+	pict_point(p, ypos + side * ytick[nyticks].tick_len, ytick[nyticks].tick_num);
+	pict_point(p, ypos, ytick[nyticks].tick_num);
 	if (side > 0.0 || marks)
-		pict_text(p, ytick[ny].tick_lab,
-			ypos - 2.0 * (1.5 * side - 0.5) * ytick[ny].tick_len,
-			ytick[ny].tick_num, 0.0, npos);
+		pict_text(p, ytick[nyticks].tick_lab,
+			ypos - 2.0 * (1.5 * side - 0.5) * ytick[nyticks].tick_len,
+			ytick[nyticks].tick_num, 0.0, npos);
 	}
+pict_point(p, ypos, miny);
 pict_point(p, ypos, xpos);
 }
 
 /* Add an x-axis to the picture pointed to by p.
- * xtick is the array of ticks to use, nxticks is the number, counting
+ * xtick is the array of ticks to use, nxticks is the number, not counting
  *	the axis label, which is the highest numbered "tick"
  * xpos is the y-coordinate of the x-axis, ypos is the x-coordinate of
  *	the y-axis: drawing ends at (ypos, xpos)
  * side = 1.0 for ticks extending up from the axis, -1.0 for down
  * marks indicates whether tick marks should be placed on the axis
  */
-static void xaxis(pict *p, tick xtick[], int nxticks, double xpos, double ypos,
-			double side, int marks)
+static void xaxis(pict *p, double minx, double maxx, tick xtick[], int nxticks,
+			double xpos, double ypos, double side, int marks)
 {
 int nx;		/* index to xtick array */
-char lpos[3];	/* position string for axis label for pict_text */
-char tpos[3];	/* position string for numerical labels for pict_text */
+char tpos[4];	/* position string for axis label for pict_text */
+char npos[3];	/* position string for numerical labels for pict_text */
 double ylaboff;	/* y offset for label */
 int lab1len;	/* length of one numerical label */
 int labslen;	/* max length of numerical labels */
 
-strcpy(lpos, "lt");	/* default is left, top of label at tick */
-strcpy(tpos, "ct");	/* default is centered, top of label at tick */
-nx = nxticks - 1;	/* index of highest element of xtick[] */
-for (labslen = 0, nx = 0; nx < nxticks - 1; nx++)
+strcpy(tpos, "lt ");	/* default for axis label is left, top of label at tick */
+strcpy(npos, "ct");	/* default for tick label is centered, top of label at tick */
+for (labslen = 0, nx = 0; nx < nxticks; nx++)
 	{		/* find maximum string length of numerical labels */
 	lab1len = strlen(xtick[nx].tick_lab);
 	if (lab1len > labslen)
 		labslen = lab1len;
 	}
-if (nx >= 1 && xtick[0].tick_num < xtick[nx].tick_num &&
-                xtick[nx].tick_num < xtick[nx - 1].tick_num)
+if (minx <= xtick[nxticks].tick_num && xtick[nxticks].tick_num <= maxx)
 	{	/* if the axis label is going on the side, not the end */
-	lpos[0] = 'c';	/* center the text */
+	tpos[0] = 'c';	/* center the text */
 	if (side < 0.0)
 		{
-		lpos[1] = 'b';
 		tpos[1] = 'b';
+		npos[1] = 'b';
 		}
-	ylaboff = p->pict_fs * xtick[nx].tick_len;	/* make room for label */
+	ylaboff = p->pict_fs * xtick[nxticks].tick_len;	/* make room for label */
 	}
-else if (xtick[nx].tick_num <= xtick[0].tick_num)	/* axis label at left end */
+else if (xtick[nxticks].tick_num < minx)	/* axis label at left end */
 	{
-	strcpy(lpos, "rm");
+	strcpy(tpos, "rm ");
 	ylaboff = 0.0;
 	}
 else							/* axis label at right end */
 	{
-	strcpy(lpos, "lm");
+	strcpy(tpos, "lm ");
 	ylaboff = 0.0;
 	}
 if (side > 0.0 || marks)	/* this is the axis label, but only label if ticking */
-	pict_text(p, xtick[nx].tick_lab, xtick[nx].tick_num,
-				xpos - side * ylaboff, 0.0, lpos);
-while (--nx >= 0)	/* for the real ticks */
+	pict_text(p, xtick[nxticks].tick_lab, xtick[nxticks].tick_num,
+				xpos - side * ylaboff, 0.0, tpos);
+pict_point(p, maxx, xpos);
+while (--nxticks >= 0)	/* for the real ticks */
 	{
-	pict_point(p, xtick[nx].tick_num, xpos);
+	pict_point(p, xtick[nxticks].tick_num, xpos);
 	if (side > 0.0 || marks)
-		pict_point(p, xtick[nx].tick_num, xpos + side * xtick[nx].tick_len);
-	pict_point(p, xtick[nx].tick_num, xpos);
+		pict_point(p, xtick[nxticks].tick_num, xpos + side * xtick[nxticks].tick_len);
+	pict_point(p, xtick[nxticks].tick_num, xpos);
 	if (side > 0.0 || marks)
-		pict_text(p, xtick[nx].tick_lab, xtick[nx].tick_num,
-			xpos - 2.0 * side * xtick[nx].tick_len, 0.0, tpos);
+		pict_text(p, xtick[nxticks].tick_lab, xtick[nxticks].tick_num,
+			xpos - 2.0 * side * xtick[nxticks].tick_len, 0.0, npos);
 	}
+pict_point(p, minx, xpos);
 pict_point(p, ypos, xpos);	/* end up at ypos, xpos */
 }
 
 /* Make axes for the picture *p:
  * xtick[], nxticks, ytick[], nyticks are arrays and counts of ticks
+ *  counts in
  * style indicates which axes should appear and where
  * bpos is position of lower end of y-axis
  * lpos is position of left end of x-axis
  * tpos is position of top end of y-axis
  * rpos is position of right end of x-axis
  */
-void pict_axes(pict *p, tick xtick[], int nxticks, tick ytick[], int nyticks,
+void pict_axes(pict *p, double minx, double maxx, tick xtick[], int nxticks,
+		double miny, double maxy, tick ytick[], int nyticks,
 		char style[], double bpos, double lpos, double tpos, double rpos)
 {
 double xpos, ypos;	/* y-coordinate of x-axis, x-coordinate of y-axis */
@@ -253,10 +235,10 @@ if (style[0] != '=' && style[1] != '=' &&
 		{
 	case '-':
 	case '0':
-		xaxis(p, xtick, nxticks, xpos, ypos, 1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, xpos, ypos, 1.0, tmarks);
 		break;
 	case '+':
-		xaxis(p, xtick, nxticks, xpos, ypos, -1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, xpos, ypos, -1.0, tmarks);
 		break;
 	case 'n':
 		break;
@@ -265,10 +247,10 @@ if (style[0] != '=' && style[1] != '=' &&
 		{
 	case '-':
 	case '0':
-		yaxis(p, ytick, nyticks, xpos, ypos, 1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, xpos, ypos, 1.0, rmarks);
 		break;
 	case '+':
-		yaxis(p, ytick, nyticks, xpos, ypos, -1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, xpos, ypos, -1.0, rmarks);
 		break;
 	case 'n':
 		break;
@@ -281,16 +263,16 @@ else if (style[0] != '=' && style[0] != '#')
 	case '-':
 		xpos = bpos;
 	case '0':
-		yaxis(p, ytick, nyticks, xpos, lpos, 1.0, rmarks);
-		xaxis(p, xtick, nxticks, xpos, lpos, 1.0, tmarks);
-		xaxis(p, xtick, nxticks, xpos, rpos, 1.0, tmarks);
-		yaxis(p, ytick, nyticks, xpos, rpos, -1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, xpos, lpos, 1.0, rmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, xpos, lpos, 1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, xpos, rpos, 1.0, tmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, xpos, rpos, -1.0, rmarks);
 		break;
 	case '+':
-		yaxis(p, ytick, nyticks, tpos, lpos, 1.0, rmarks);
-		xaxis(p, xtick, nxticks, tpos, lpos, -1.0, tmarks);
-		xaxis(p, xtick, nxticks, tpos, rpos, -1.0, tmarks);
-		yaxis(p, ytick, nyticks, tpos, rpos, -1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, tpos, lpos, 1.0, rmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, tpos, lpos, -1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, tpos, rpos, -1.0, tmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, tpos, rpos, -1.0, rmarks);
 		break;
 	case 'n':
 		fputs("(axes) Can't have double y-axes and no x-axis.\n", dap_err);
@@ -304,16 +286,16 @@ else if (style[1] != '=' && style[1] != '#')
 	case '-':
 		ypos = lpos;
 	case '0':
-		xaxis(p, xtick, nxticks, bpos, ypos, 1.0, tmarks);
-		yaxis(p, ytick, nyticks, bpos, ypos, 1.0, rmarks);
-		yaxis(p, ytick, nyticks, tpos, ypos, 1.0, rmarks);
-		xaxis(p, xtick, nxticks, tpos, ypos, -1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, bpos, ypos, 1.0, tmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, bpos, ypos, 1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, tpos, ypos, 1.0, rmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, tpos, ypos, -1.0, tmarks);
 		break;
 	case '+':
-		xaxis(p, xtick, nxticks, bpos, rpos, 1.0, tmarks);
-		yaxis(p, ytick, nyticks, bpos, rpos, -1.0, rmarks);
-		yaxis(p, ytick, nyticks, tpos, rpos, -1.0, rmarks);
-		xaxis(p, xtick, nxticks, tpos, rpos, -1.0, tmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, bpos, rpos, 1.0, tmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, bpos, rpos, -1.0, rmarks);
+		yaxis(p, miny, maxy, ytick, nyticks, tpos, rpos, -1.0, rmarks);
+		xaxis(p, minx, maxx, xtick, nxticks, tpos, rpos, -1.0, tmarks);
 		break;
 	case 'n':
 		fputs("(axes) Can't have double x-axes and no y-axis.\n", dap_err);
@@ -322,10 +304,78 @@ else if (style[1] != '=' && style[1] != '#')
 	}
 else
 	{	/* totally boxed */
-	yaxis(p, ytick, nyticks, bpos, lpos, 1.0, rmarks);
-	xaxis(p, xtick, nxticks, bpos, rpos, 1.0, tmarks);
-	yaxis(p, ytick, nyticks, tpos, rpos, -1.0, rmarks);
-	xaxis(p, xtick, nxticks, tpos, lpos, -1.0, tmarks);
+	yaxis(p, miny, maxy, ytick, nyticks, bpos, lpos, 1.0, rmarks);
+	xaxis(p, minx, maxx, xtick, nxticks, bpos, rpos, 1.0, tmarks);
+	yaxis(p, miny, maxy, ytick, nyticks, tpos, rpos, -1.0, rmarks);
+	xaxis(p, minx, maxx, xtick, nxticks, tpos, lpos, -1.0, tmarks);
+	}
+}
+
+#define AXISMARGIN 0.05
+#define RADIUSFACTOR 100.0
+
+/* round x to number of places indicated by ndigs,
+ * up or down indicated by updown (= 1 for up, 0 for nearest, -1 for down)
+ */
+static double rnd(double x, int ndigs, int updown)
+{
+double tx;
+double pow10;
+double precision;
+
+if (x == 0.0)
+	return 0.0;
+for (precision = 1.0; ndigs > 0; ndigs--)
+	precision *= 10.0;
+tx = x;		/* so we don't have to worry about the sign */
+if (tx < 0.0)
+	tx = -x;
+if (tx >= 1.0)
+	{
+	for (pow10 = 1.0; pow10 < tx; pow10 *= 10.0)
+		; 
+	}
+else
+	{
+	for (pow10 = 1.0; 1.0 / pow10 > tx; pow10 *= 10.0)
+		;
+	pow10 /= 10.0;
+	}
+pow10 = precision / pow10;
+if (pow10 < 1.0)
+	pow10 = 1.0;
+if (updown == 1)
+	return ceil(x * pow10) / pow10;
+else if (updown == -1)
+	return floor(x * pow10) / pow10;
+else
+	return rint(x * pow10) / pow10;
+}
+
+/* make label; OK this can crash the eventual sprintf if you have some really,
+ * really big number with more than dap_maxtxt-1 digits, but then your picture
+ * wouldn't look so good either.
+ */
+static void makeform(char form[], double max, int ndigs)
+{
+int ndec;	/* number of decimal places */
+double scale;	/* power of 10 to compare max to */
+
+if (ndigs > 9)
+	ndigs = 9;
+max = fabs(max);
+strcpy(form, "%.0f");
+if (max == 0.0)		/* this really shouldn't happen */
+	sprintf(form, "%%.%dg", ndigs);
+else
+	{
+	for (scale = 1.0; ndigs > 1; ndigs--)
+		scale *= 10.0;
+	for (ndec = 0; max < scale; max *= 10.0)
+		ndec++;
+	if (ndec > 9)
+		ndec = 9;
+	form[2] += ndec;
 	}
 }
 
@@ -333,10 +383,10 @@ else
 static void ticks(
 	tick ticks[],		/* array of ticks to make, allocated by caller */
 	double min,		/* coordinate of lowest valued tick */
-	double space,		/* space between ticks */
+	double max,		/* coordinate of highest valued tick */
+	int ndigs,		/* number of digits for tick label */
 	double ticklen,		/* length of tick line */
-	char *form,		/* format string for numerical labels */
-	int nticks,		/* number of ticks requested */
+	int nticks,		/* number of ticks requested not including axis label */
 	double labpos,		/* position of axis label */
 	char *alab,		/* axis label */
 	double (*tfunct)()	/* function for adjusting printed values at ticks */
@@ -345,21 +395,41 @@ static void ticks(
 int n;
 static char *lab;
 double coord;
+double space;
+double tcoord;		/* coord or transformed coord */
+char form[5];		/* format string */
+double tmin, tmax;	/* (possibly) transformed min, max */
 
-if (!form)
-	{
-	fputs("(ticks) No format.\n", dap_err);
-	exit(1);
-	}
 if (!lab)
 	lab = dap_malloc(dap_maxtxt + 1, "dap_maxtxt");
-for (n = 0; n < nticks - 1; n++)
+if (nticks > 1)
+	space = (max - min) / (double) (nticks - 1);
+else
+	space = 0;
+if (tfunct)
+	tmin = tfunct(min);
+else
+	tmin = min;
+tmin = fabs(tmin);
+if (tfunct)
+	tmax = tfunct(max);
+else
+	tmax = max;
+tmax = fabs(tmax);
+if (tmin > tmax)
+	tmax = tmin;
+makeform(form, tmax, ndigs);
+for (n = 0; n < nticks; n++)
 	{
-	coord = min + space * (double) n;
-	if (tfunct)
-		sprintf(lab, form, (*tfunct)(coord));
+	if (nticks > 1)
+		coord = min + space * (double) n;
 	else
-		sprintf(lab, form, coord);
+		coord = (min + max) / 2.0;
+	if (tfunct)
+		tcoord = (*tfunct)(coord);
+	else
+		tcoord = coord;
+	sprintf(lab, form, tcoord);
 	pict_maketick(ticks + n, coord, lab, ticklen);
 	}
 pict_maketick(ticks + n, labpos, alab, ticklen);
@@ -378,32 +448,32 @@ double pict_autoaxes(pict *p, char *xlab, char *ylab, char *axspec,
 {
 pict *pp;				/* for stepping through array of picts */
 int totpts;				/* total number of points */
-double xspace, yspace;
-static double xlen, ylen;
-static double minx, maxx, miny, maxy;	/* min, max values of x, y in picts */
-double minx1, maxx1;
-static int nxticks, nyticks;
-tick *xticks;
-tick *yticks;
-double xticklen;
-double yticklen;
-char xform[10], yform[10];
-char as[3];
-double lpos, rpos, bpos, tpos;
+double minx, maxx, miny, maxy;		/* min, max values of x, y in picts */
+double minxt, maxxt, minyt, maxyt;	/* min, max tick values */
+int nxticks, nyticks;			/* number of ticks, for x, y */
+tick *xticks;				/* tick array for x */
+tick *yticks;				/* tick array for y */
+double xticklen;			/* length of tick mark for x */
+double yticklen;			/* length of tick mark for y */
+char as[3];				/* axis specifications, from axspec */
+double lpos, rpos, bpos, tpos;		/* axis positions: left, right ends of x-axis,
+					 * bottom, top ends of y-axis
+					 */
 double xlabpos, ylabpos;	/* coordinates of position of x, y axis labels */
-double captoff;
-double width, height;
-int lexpand, rexpand;
-double specxmax, specxmin;
-double specymax, specymin;
-int specxticks, specyticks;
-int a, w;		/* indexs to axspec, word */
-char *word;
-double digs, places, sign;
-int nt;
-double xscale, yscale;	/* to compensate for scaling in label positions */
-int xlablines;		/* number lines in xlabel */
+double captoff;			/* offset into caption */
+double width, height;		/* width, height of graph in points */
+double specxmax, specxmin;	/* specified min, max for x */
+double specymax, specymin;	/* specified min, max for y */
+int specxticks, specyticks;	/* specified number of ticks, for x, y */
+int nxdigs, nydigs;		/* number of digits for tick labels, for x, y */
+int a, w;			/* indexs to axspec, word */
+char *word;			/* word extracted from axspec */
+double digs, places, sign;	/* for computing max, min numbers */
+int nt;				/* number of ticks */
+double xscale, yscale;		/* to compensate for scaling in label positions */
+int xlablines;			/* number lines in xlabel */
 
+/* start out with centered around 0.0 */
 minx = 0.0;
 maxx = 0.0;
 miny = 0.0;
@@ -414,41 +484,53 @@ bpos = 0.0;
 tpos = 0.0;
 xlabpos = 0.0;
 ylabpos = 0.0;
+/* no scaling yet */
 xscale = 1.0;
 yscale = 1.0;
 captoff = 0.0;
+/* allocate tick arrays */
 xticks = (tick *) dap_malloc(sizeof(tick) * dap_maxntxt, "dap_maxntxt");
 yticks = (tick *) dap_malloc(sizeof(tick) * dap_maxntxt, "dap_maxntxt");
+/* default is nothing specified: make NaNs, -1 */
 specxmin = 0.0 / 0.0;
 specxmax = 0.0 / 0.0;
 specymin = 0.0 / 0.0;
 specymax = 0.0 / 0.0;
 specxticks = -1;
 specyticks = -1;
+nxdigs = 3;
+nydigs = 3;
+/* allocate word */
 word = dap_malloc(dap_namelen + 1, "");
+/* now parse the axis specifications, if present */
 if (axspec && axspec[0])
 	{
+	/* first skip blanks */
 	for (a = 0; axspec[a] == ' '; a++)
 		;
+	/* if there are axis formats */
 	if (axspec[a] == '-' || axspec[a] == '+' || axspec[a] == '0' ||
 			axspec[a] == 'n' || axspec[a] == '=' ||
 			axspec[a] == '#')
 		{
+		/* copy to as */
 		as[0] = axspec[a++];
 		if (axspec[a] == '-' || axspec[a] == '+' || axspec[a] == '0' ||
 				axspec[a] == 'n' || axspec[a] == '=' ||
 				axspec[a] == '#')
 			as[1] = axspec[a++];
-		else
+		else	/* no second char, make '0' as default */
 			as[1] = '0';
 		}
-	else
+	else	/* otherwise, use default */
 		strcpy(as, "00");
 	as[2] = '\0';
+	/* now look for other words */
 	while (axspec[a] == ' ')
 		a++;
 	while (axspec[a])
 		{
+		/* extract one word */
 		for (w = 0; axspec[a] && axspec[a] != ' '; )
 			{
 			if (w < dap_namelen)
@@ -457,7 +539,7 @@ if (axspec && axspec[0])
 				{
 				word[w] = '\0';
 				fprintf(dap_err,
-			"pict_autoaxes: word in axspec too long: %s\n",
+			"(pict_autoaxes) word in axspec too long: %s\n",
 					word);
 				exit(1);
 				}
@@ -466,13 +548,16 @@ if (axspec && axspec[0])
 		if (!strncmp(word, "MAXX", 4) || !strncmp(word, "MINX", 4) ||
 				!strncmp(word, "MAXY", 4) || !strncmp(word, "MINY", 4))
 			{
+			/* for max, min spec, position at next char for number */
 			w = 4;
+			/* default is positive, see if negative */
 			sign = 1.0;
 			if (word[w] == '-')
 				{
 				sign = -1.0;
 				w++;
 				}
+			/* standard decimal conversion, allowing for dec pt */
 			for (digs = 0.0, places = 0.0;
 					('0' <= word[w] && word[w] <= '9') ||
 					word[w] == '.'; w++)
@@ -482,7 +567,7 @@ if (axspec && axspec[0])
 					if (places > 0.0)
 						{
 						fprintf(dap_err,
-				"pict_autoaxes: bad number for MIN or MAX: %s\n",
+				"(pict_autoaxes) bad number for MIN or MAX: %s\n",
 							word + 4);
 						exit(1);
 						}
@@ -499,13 +584,15 @@ if (axspec && axspec[0])
 			digs *= sign;
 			if (places > 0.0)
 				digs /= places;
+			/* should be space after number */
 			if (word[w] && word[w] != ' ')
 				{
 				fprintf(dap_err,
-				"pict_autoaxes: bad number for MIN or MAX: %s\n",
+				"(pict_autoaxes) bad number for MIN or MAX: %s\n",
 					word + 3);
 				exit(1);
 				}
+			/* mAx or mIn, m.X or m.Y */
 			if (word[1] == 'A')
 				{
 				if (word[3] == 'X')
@@ -521,38 +608,67 @@ if (axspec && axspec[0])
 					specymin = digs;
 				}
 			}
+		/* now if was tick spec instead */
 		else if (!strncmp(word, "NXTICKS", 7) ||
 				!strncmp(word, "NYTICKS", 7))
 			{
+			/* integer conversion */
 			for (nt = 0, w = 7; '0' <= word[w] && word[w] <= '9'; w++)
 				nt = 10 * nt + word[w] - '0';
-			if (word[w] && word[w] != ' ' || !nt)
+			/* should have number, should end in space */
+			if (word[w] && word[w] != ' ')
 				{
 				fprintf(dap_err,
-				"pict_autoaxes: bad number of ticks: %s\n",
+				"(pict_autoaxes) bad number of ticks: %s\n",
 					word + 7);
 				exit(1);
 				}
+			/* nXticks or nYticks */
 			if (word[1] == 'X')
-				specxticks = nt + 1;
+				specxticks = nt;
 			else
-				specyticks = nt - 1;
+				specyticks = nt;
 			}
-		else
+		/* now if specifying label precision */
+		else if (!strncmp(word, "NXDIGITS", 8) ||
+				!strncmp(word, "NYDIGITS", 8))
+			{
+			/* integer conversion */
+			for (nt = 0, w = 8; '0' <= word[w] && word[w] <= '9'; w++)
+				nt = 10 * nt + word[w] - '0';
+			/* should have number, should end in space */
+			if (word[w] && word[w] != ' ')
+				{
+				fprintf(dap_err,
+				"(pict_autoaxes) bad number of digits: %s\n",
+					word + 8);
+				exit(1);
+				}
+			/* nXdigits or nYdigits */
+			if (word[1] == 'X')
+				nxdigs = nt;
+			else
+				nydigs = nt;
+			}
+		else	/* tried to match and failed */
 			{
 			fprintf(dap_err,
-			"pict_autoaxes: bad axes specification: %s\n",
+			"(pict_autoaxes) bad axes specification: %s\n",
 				word);
 			exit(1);
 			}
+		/* and on to the next */
 		while (axspec[a] == ' ')
 			a++;
 		}
 	}
-else
+else	/* the default */
 	strcpy(as, "00");
+/* now we find the min, max for x, y for all the points in the pict
+ * and check that there are indeed points
+ */
 for (pp = p, totpts = 0; pp && pp->pict_next; pp = pp->pict_next)
-	{			/* find min, max of x, y in all the picts */
+	{	/* step through linked list of picts */
 	if (pp->pict_npts)
 		{
 		if (pp == p || minx > pp->pict_minx)
@@ -568,125 +684,75 @@ for (pp = p, totpts = 0; pp && pp->pict_next; pp = pp->pict_next)
 	}
 if (!totpts)
 	{
-	fputs("pict_autoaxes: no points.\n", dap_err);
+	fputs("(pict_autoaxes) no points.\n", dap_err);
 	exit(1);
 	}
+/* the specs were set to NaNs, so if finite, they were present */
 if (finite(specxmin))
 	minx = specxmin;
+else	/* we like to make minx leave a little extra room and be a nice decimal;
+	 * note that we are not dealing with xfunct and yfunct, because we have
+	 * no inverse functions for them: the caller will just have to deal with those.
+	 */
+	{
+	/* first make sure it includes 0 */
+	if (minx > 0.0)
+		minx = 0.0;
+	}
 if (finite(specxmax))
 	maxx = specxmax;
+else
+	{
+	if (maxx < 0.0)
+		maxx = 0.0;
+	}
+minxt = minx;
+minx -= AXISMARGIN * (maxx - minx);
+minx = rnd(minx, nxdigs, -1);
+maxxt = maxx;
+maxx += AXISMARGIN * (maxx - minx);
+maxx = rnd(maxx, nxdigs, 1);
 if (finite(specymin))
 	miny = specymin;
+else
+	{
+	if (miny > 0.0)
+		miny = 0.0;
+	}
 if (finite(specymax))
 	maxy = specymax;
-minx1 = minx;
-maxx1 = maxx;
-if (minx > 0.0 && !finite(specxmin))
-	minx = 0.0;
-if (miny > 0.0 && !finite(specymin))
-	miny = 0.0;
-if (maxx < 0.0 && !finite(specxmax))
-	maxx = 0.0;
-if (maxy < 0.0 && !finite(specymax))
-	maxy = 0.0;
-xlen = 1e5 / (maxx - minx);
-if (!finite(xlen))
-	{
-	fprintf(dap_err,
-	"pict_autoaxes: minimum (%g) and maximum (%g) of x values are equal.\n",
-		minx, maxx);
-	exit(1);
-	}
-if (!finite(specxmin))
-	minx = arintdown(xlen * minx) / xlen;
-if (!finite(specxmax))
-	maxx = arintup(xlen * maxx) / xlen;
-ylen = 1e5 / (maxy - miny);
-if (!finite(ylen))
-	{
-	fprintf(dap_err,
-	"pict_autoaxes: minimum (%g) and maximum (%g) of y values are equal.\n",
-		miny, maxy);
-	exit(1);
-	}
-if (!finite(specymin))
-	miny = arintdown(ylen * miny) / ylen;
-if (!finite(specymax))
-	maxy = arintup(ylen * maxy) / ylen;
-xlen = maxx - minx;
-if (specxticks > 2)
-	nxticks = specxticks - 2;
 else
-	nxticks = 10.0;
-if (xlen >= 1.0)
-        {
-        for (xspace = 1.0; nxticks * xspace < xlen; xspace *= nxticks)
-                ; 
-        xspace *= ceil(xlen / xspace) / nxticks;
-        }
-else            
-        {
-        for (xspace = 0.1; 0.1 * xspace > xlen; xspace *= 0.1)
-                ;
-        xspace *= ceil(xlen / xspace) / 10.0;
-        }
-ylen = maxy - miny;
-if (specyticks > 0)
+	{
+	if (maxy < 0.0)
+		maxy = 0.0;
+	}
+minyt = miny;
+if (miny != 0.0)
+	miny -= AXISMARGIN * (maxy - miny);
+miny = rnd(miny, nydigs, -1);
+maxyt = maxy;
+maxy += AXISMARGIN * (maxy - miny);
+maxy = rnd(maxy, nydigs, 1);
+if (specxticks >= 0)
+	nxticks = specxticks;
+else
+	nxticks = 11;
+if (nxticks > dap_maxntxt)
+	{
+	fprintf(dap_err, "(pict_autoaxes) Too many x-ticks (%d)\n", nxticks);
+	exit(1);
+	}
+if (specyticks >= 0)
 	nyticks = specyticks;
 else
-	nyticks = 10.0;
-if (ylen >= 1.0)
-        {
-        for (yspace = 1.0; nyticks * yspace < ylen; yspace *= nyticks)
-                ;
-        yspace *= ceil(ylen / yspace) / nyticks;
-        }
-else
-        {
-        for (yspace = 0.1; 0.1 * yspace > ylen; yspace *= 0.1)
-                ;
-        yspace *= ceil(ylen / yspace) / 10.0;
-        }
-if (xspace == rint(xspace))
-	strcpy(xform, "%.0f");
-else if (10.0 * xspace == rint(10.0 * xspace))
-	strcpy(xform, "%.1f");
-else
-	strcpy(xform, "%.2f");
-if (yspace == rint(yspace))
-	strcpy(yform, "%.0f");
-else if (10.0 * yspace == rint(10.0 * yspace))
-	strcpy(yform, "%.1f");
-else
-	strcpy(yform, "%.2f");
-if (minx < minx1)
-	minx = xspace * floor(minx / xspace);
-else
-	minx = xspace * (floor(minx / xspace) - 1.0);
-lexpand = (minx + xspace <= minx1);
-	miny = yspace * floor(miny / yspace);
-if (maxx > maxx1)
-	maxx = xspace * ceil(maxx / xspace);
-else
-	maxx = xspace * (ceil(maxx / xspace) + 1.0);
-rexpand = (maxx - xspace >= maxx1);
-maxy = yspace * ceil(maxy / yspace);
+	nyticks = 11;
+if (nyticks > dap_maxntxt)
+	{
+	fprintf(dap_err, "(pict_autoaxes) Too many y-ticks (%d)\n", nyticks);
+	exit(1);
+	}
 xticklen = 2.0;
 yticklen = 2.0;
-if (specxticks > 0)
-	{
-	nxticks = 2 + specxticks - lexpand - rexpand;
-	xspace = (maxx - minx) / ((double) specxticks);
-	}
-else
-	nxticks = 2 + (int) rint((maxx - minx) / xspace) - lexpand - rexpand;
-if (specyticks > 0)
-	{
-	nyticks = 2 + specyticks;
-	yspace = (maxy - miny) / ((double) specyticks);
-	}
-else
-	nyticks = 2 + (int) rint((maxy - miny) / yspace);
 if (autopos)
 	{
 	if (autopos == PORTRAIT)
@@ -772,28 +838,23 @@ for (w = 0, xlablines = 0; xlab[w]; w++)
 		xlablines++;
 	}
 captoff += 1.4 * xlablines * p->pict_fs / yscale;
-if (lexpand)
-	ticks(xticks, minx + xspace, xspace, xticklen / yscale,
-			xform, nxticks, xlabpos, xlab, xfunct);
-else
-	ticks(xticks, minx, xspace, xticklen / yscale,
-			xform, nxticks, xlabpos, xlab, xfunct);
-ticks(yticks, miny, yspace, yticklen / xscale,
-		yform, nyticks, ylabpos, ylab, yfunct);
-pict_axes(pp, xticks, nxticks, yticks, nyticks, as, bpos, lpos, tpos, rpos);
+ticks(xticks, minxt, maxxt, nxdigs, xticklen / yscale,
+		nxticks, xlabpos, xlab, xfunct);
+ticks(yticks, minyt, maxyt, nydigs, yticklen / xscale,
+		nyticks, ylabpos, ylab, yfunct);
+pict_axes(pp, minx, maxx, xticks, nxticks,
+		miny, maxy, yticks, nyticks, as, bpos, lpos, tpos, rpos);
 pict_text(pp, caption, 0.5 * (minx + maxx), miny - captoff, 0.0, "ct ");
 if (autopos)
 	{
 	pict_scale(p, 0.5 * (minx + maxx), 0.5 * (miny + maxy), xscale, yscale);
 	pict_translate(p, LEFT + 0.5 * (width - (minx + maxx)),
 			BOTTOM + 0.5 * (height - (miny + maxy)));
-	xspace *= width / (maxx - minx);
-	yspace *= width / (maxy - miny);
 	}
 dap_free(xticks);
 dap_free(yticks);
 dap_free(word);
-return sqrt(xspace * xspace + yspace * yspace);
+return sqrt((width * width + height * height) / RADIUSFACTOR);
 }
 
 /* this stuff is just for reference */
